# User defined functions for Resistivity in Less Than Three Dimensions

# Returns a python list of floats from a delimited string. If an 'empty' value exists,
# the NoneType will be substituted; i.e. '4\t\t5\t6' will return [4.0,None,5.0,6.0] if
# '\t' is the delimiter. This is to differentiate between zero and an empty value.
def readline (str,delimiter='\t'):
	val = []
	for ele in str.split(delimiter):
		try: val.append(float(ele))
		except ValueError: val.append(None)
	return val

# Reads in a delimited data file and separates the data into a 2-dim array (table). 
# Column #s are calculated by the first line in the file and Row #s by the number of lines.
# If initial values not set on first line of data, zero is assumed. Returns a numpy.ndarray.
def readdata (filepath):
	from numpy import array, arange, empty, zeros
	# open file at path in read-mode and read lines into an array
	lines = array(open(filepath,'r').readlines())
	# set initial values to zero (really used to store last 'good' values)
	initial = zeros(len(readline(lines[0])))
	# create table (values are random unlike using zero())
	table = empty((len(lines),len(initial)))
	
	count = 0 # init counter and begin filling table
	for line in lines:
		val = readline(line) # read values for current line
		for j in arange(len(val)):
			# if values exist, set as new initials
			if val[j] != None: initial[j] = val[j]
		table[count] = initial # update table
		count+=1
	
	return table

# Similar to readdata() being called on every file in the directory, except the positions
# for all but the last probe are read only once from the first line of the first file. 
# 
# Returns two numpy.ndarrays: a 1-d containing positions of the first 3 probes, and a 2-d
# containing P- (x,y) positions and their corresponding voltages. This should be less memory
# demanding than iterating with readdata() since the first 3 probes are never changed in
# the contour data set and need not be kept for every voltage reading. 
#
# The order Python reads files is assumed to be determined by your OS sorting rules. To
# ensure positions are read, the first line of all data files should have position values.
def getcontourdata (directory):
	from os import listdir
	from os.path import join, isfile
	from numpy import array, empty, sum
	# open all files in the directory and read all lines into file-specific lists.
	# i.e. 'files' is a python list of 1-dim numpy.ndarrays each representing a file
	# and containing all the lines (as strings) for that particular file.
	files = [array(open(join(directory, file),'r').readlines()) \
		for file in listdir(directory) \
		if isfile(join(directory, file))]
	
	# read in all position values from the first line of the first file
	initial = readline(files[0][0])[0:-1]
	
	# array for storing P- (x,y) and voltages. row number is the total rows of all files
	V = empty((sum(len(file) for file in files),3))
	
	count = 0
	for file in files: # iterate through files
		for line in file: # iterate through lines in focused file
			vals = readline(line)
			# set x,y values if they exist
			if vals[-3] != None: initial[-2] = vals[-3]
			if vals[-2] != None: initial[-1] = vals[-2]
			# update V array
			V[count,0] = initial[-2]
			V[count,1] = initial[-1]
			V[count,2] = vals[-1]
			count+=1
	
	# return [I+x,I+y,I-x,I-y,P+x,P+y], [P-x, P-y, Voltage]
	return initial[0:-2],V

# Return 2-dim arrays for x, y, and voltage mapped to a grid, ready for contour plotting.
# Resolution is the number of pieces a "unit" will be split into. Example: range(0,12) will
# split into 12*res so that (0,1) has res pieces. If range is too small, eg (0,0.2) or (0,2),
# upscaling will occur such that the minimum pieces over the entire range is 10*res
# Note: large resolutions (eg 100) may throw a MemoryError.
def prepcontourdata (data,resolution=10):
	# finds range of x,y and ratio that ensures grid is comprised of squares
	dx = data[:,0].max()-data[:,0].min()
	dy = data[:,1].max()-data[:,1].min()
	if dy == 0 or dx == 0: raise NameError('All x\'s or y\'s are zero')
	else: ratio = dy/dx
	
	# upscale dx if too small
	while dx < 10: dx *= 10
	resolution = dx*resolution
	
	# create grid from min/max, separated into evenly spaced squares
	from numpy import linspace
	x = linspace(data[:,0].min(),data[:,0].max(),resolution)
	y = linspace(data[:,1].min(),data[:,1].max(),ratio*resolution)
	
	# map data onto grid
	from matplotlib.mlab import griddata
	z = griddata(data[:,0],data[:,1],data[:,2],x,y,'nn')
	
	return x,y,z

# Finds the difference between I+ and the two P probes, and returns the lower.
def geta (data):
	from numpy import sqrt, sum
	a1 = sqrt((data[:,0]-data[:,4])**2 + (data[:,1]-data[:,5])**2)/100
	a2 = sqrt((data[:,0]-data[:,6])**2 + (data[:,1]-data[:,7])**2)/100
	if sum(a1) <= sum(a2): return a1
	else: return a2

def getb (data):
	from numpy import sqrt
	return sqrt((data[:,4]-data[:,6])**2 + (data[:,5]-data[:,7])**2)/100

# Returns Schlumberger or Wenner or ...
def getmethod (meth):
	if meth == 'S': return 'Schlumberger'
	elif meth == 'W': return 'Wenner'
	else: return None

# Returns from filename the dimension (int), method (str), I (float), and id (str) .
# Requires filename to be 'nD-M-f.f ##.txt' where n = dim, M = method, f.f = current,
# and ## = some identifier (such as the run). There _is_ a space between f.f and ##.
# Valid filenames include: '1D-S-1.2 12.txt' (recommended), '1D-1.2 12.txt', '1D-1.2.txt'
def parsefilename (str):
	val  = str.split('-')
	# try setting values, set None if assignment not possible
	
	try: dim = int(val[0][0])
	except ValueError: dim = None
	
	try: meth = getmethod(val[1])
	except IndexError: meth = None
	
	try: I = float(val[-1].split(' ')[0])
	except ValueError:
		try: I = float(val[-1].rsplit('.',1)[0])
		except ValueError: I = None
	
	try: id = val[-1].split(' ')[1].split('.')[0]
	except IndexError: id = None
	
	return dim,meth,I,id

# Simply checks if string provided is a directory (folder) and creates it if not found.
# The created directory is either absolute or relative to the working directory of the
# script making the function call. Returns the input string without modification.
def createdir (str):
	from os.path import isdir
	if not isdir(str):
		from os import makedirs
		makedirs(str)
	return str
